/*
 * File: FibonacciHeap.java
 * Copyright (c) 2012, Pier Luigi Pau (pigipau@gmail.com)
 *
 * Version 0.2
 * Release: 2012-06-25
 *
 * This is an implementation of the Fibonacci Heap, a data structure
 * for priority queues with DecreaseKey support, described in:
 * Michael L. Fredman, Robert Endre Tarjan
 * Fibonacci heaps and their uses in improved network optimization
 * algorithms - 1987
 *
 * This class is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This class is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this file. If not, see <http://www.gnu.org/licenses/>.
 */

package cytoscape.util.intr;

import java.util.HashMap;

/**
 * A class implementing a priority queue as a Fibonacci Heap.
 *
 * @param T The type of elements to store in the heap.
 * @author Pier Luigi Pau (pigipau@gmail.com)
 */
public class FibonacciHeap<T> {
	
	public static final class FibonacciHNode<T> {
		private T item;
		
		private double key;
		
		private FibonacciHNode<T> leftSibling;

		private FibonacciHNode<T> rightSibling;

		private FibonacciHNode<T> parent;

		private FibonacciHNode<T> child;
		
		private boolean marked;

		private int degree;

		public FibonacciHNode(T item, double key) {
			super();
			this.item = item;
			this.key = key;
			this.leftSibling = this;
			this.rightSibling = this;
			this.parent = null;
			this.child = null;
			this.marked = false;
			this.degree = 0;
		}

		/**
		 * Returns the element contained in the node.
		 * 
		 * @return The element contained in the node.
		 */
		public T getItem() {
			return item;
		}

		/**
		 * Returns the priority associated to the element contained 
		 * in the node.
		 * 
		 * @return The priority of the node.
		 */
		public double getKey() {
			return key;
		}
		
		/**
		 * Links a sibling to this node. The sibling chains are merged,
		 * i.e. every sibling of the new sibling also becomes a sibling
		 * of the node.
		 * 
		 * @param newSibling
		 */
		private void linkSibling(FibonacciHNode<T> newSibling) {
			// puts new sibling to the left
			FibonacciHNode<T> oldSibling = newSibling.rightSibling;
			oldSibling.leftSibling = this.leftSibling;
			newSibling.rightSibling = this;
			this.leftSibling.rightSibling = oldSibling;
			this.leftSibling = newSibling;
		}
		
		/**
		 * Unlinks the node from its siblings.
		 */
		private void unlinkSiblings() {
			if (this.leftSibling != this || this.rightSibling != this) {
				this.leftSibling.rightSibling = this.rightSibling;
				this.rightSibling.leftSibling = this.leftSibling;
				this.leftSibling = this;
				this.rightSibling = this;
			}
		}

		/**
		 * Unlinks the node from its children. Returns a reference
		 * to one of its children; the other children can be reached
		 * by navigating the siblings of the returned child.
		 * If the node has no children, no action is performed and
		 * the return value is null.
		 * 
		 * @return A former child of the node, or null if it had none.
		 */
		private FibonacciHNode<T> unlinkChildren() {
			FibonacciHNode<T> oldChild = child;
			
			if (child != null) {
				do {
					child.parent = null;
					child = child.rightSibling;
				} while (child != oldChild);
				
				child = null;
				degree = 0;
			}
			
			return oldChild;
		}
		
		/**
		 * Links the parameter as a child to this node. The parameter
		 * is unlinked from its siblings and linked as a sibling to
		 * the existing children of this node.
		 * 
		 * @param newChild
		 */
		private void linkChild(FibonacciHNode<T> newChild) {
			newChild.parent = this;
			newChild.marked = false;
			newChild.unlinkSiblings();
			
			if (child != null) {
				child.linkSibling(newChild);
			}

			// always update this pointer, so that we can
			// reconstruct the reverse order of linking
			// by visiting right siblings.
			child = newChild;
			degree++;
		}
	}

	/**
	 * This reference is updated to the minimum element of the
	 * Fibonacci Heap after each operation. Additionally, the
	 * root list of the Fibonacci Heap can be accessed by
	 * navigating the siblings of the referenced node.
	 * The reference is null if and only if the Fibonacci Heap
	 * is empty.
	 */
	private FibonacciHNode<T> minPointer;

	/**
	 * Size of the Fibonacci Heap.
	 */
	private int heapSize;
	
	/**
	 * Creates an empty Fibonacci Heap.
	 */
	public FibonacciHeap() {
		super();
		minPointer = null;
		heapSize = 0;
	}

	/**
	 * Add a node and its siblings to the root list.
	 * @param entry
	 */
	private void addRoot(FibonacciHNode<T> entry) {
		if (minPointer == null) {
			minPointer = entry;
			
		} else {
			minPointer.linkSibling(entry);
		}
	}

	/**
	 * If the Fibonacci Heap is empty, the element is set as the
	 * global minimum. Otherwise, a comparison of keys is performed
	 * and the global minimum pointer is updated if the element
	 * has a lower key. The element must be in the root list already.
	 * 
	 * @param element
	 */
	private void updateMin(FibonacciHNode<T> element) {
		if (minPointer == null) {
			minPointer = element;

		} else {
			double currentMinKey = minPointer.getKey();
			double newMinKey = element.getKey();
			
			if (newMinKey < currentMinKey) {
				minPointer = element;
			}
		}
	}

	/**
	 * Iterates over the root list and find the global minimum.
	 * Called internally e.g. after deleting the global minimum.
	 * A node in the root list has to be passed as a parameter
	 * because minPointer may have been temporarily set to null.
	 * 
	 * @param source
	 */
	private void findNewMin(FibonacciHNode<T> source) {
		if (source != null) {
			FibonacciHNode<T> current = source;
			minPointer = null;
			
			do {
				updateMin(current);
				current = current.rightSibling;
			} while (current != source);
			
		}
	}

	/**
	 * Performs the heap rearrangement after a delete/deleteMin operation.
	 * After a rearrangement, each node in the root list has a unique degree.
	 *
	 * @param source A node in the root list
	 */
	private void rearrange(FibonacciHNode<T> source) {
		// find siblings with the same degree
		boolean foundLinkees;
		FibonacciHNode<T> firstNode = source;
		
		HashMap<Integer, FibonacciHNode<T>> visNodes = new HashMap<Integer, FibonacciHNode<T>>();
		
		do {
			// new iteration, new search for pairs with same degree
			foundLinkees = false;
			visNodes.clear();
			
			// iterate over root list
			FibonacciHNode<T> current = firstNode;
			FibonacciHNode<T> linkeeA = null, linkeeB = null;
			
			if (current != null) {
				int cDegree, lDegree = -1;
				do {
					cDegree = current.degree;

					// have we found another node with the same degree yet?
					if (visNodes.containsKey(cDegree)) {
						// remember the least degree for which we found pairs
						if (lDegree < 0 || lDegree > cDegree) {
							lDegree = cDegree;
							linkeeB = current;
							linkeeA = visNodes.get(cDegree);
						}
					
					} else {
						// take note of having found a node with this degree
						visNodes.put(cDegree, current);
					}
					
					// iterate over root list
					current = current.rightSibling;
				} while (current != firstNode);
				
				if (linkeeA != null) {
					// we have found a pair
					do {
						// compare keys and choose which node should be linked
						// as the parent, and which node as the child
						FibonacciHNode<T> nodeParent, nodeChild;
						
						if ( linkeeA.getKey() < linkeeB.getKey() ) {
							nodeParent = linkeeA;
							nodeChild = linkeeB;
						} else {
							nodeParent = linkeeB;
							nodeChild = linkeeA;
						}
						
						// set flag that we have found a pair
						foundLinkees = true;

						// unlink nodeChild from its siblings
						// and add it as a child of nodeParent
						nodeChild.unlinkSiblings();
						nodeParent.linkChild(nodeChild);
						
						// we have to make sure that firstNode links to a node
						// in the root list; if we have linked firstNode as a child,
						// set the firstNode reference to its new parent
						if (nodeChild == firstNode) {
							firstNode = nodeParent;
						}
						
						// iterate over degrees: if we have created a new node
						// of the current degree + 1, and there is already such
						// a node, we have to link them
						lDegree++;
						if (visNodes.containsKey(lDegree)) {
							linkeeA = visNodes.get(lDegree);
							linkeeB = nodeParent;
						} else {
							linkeeA = null;
						}
						
					} while (linkeeA != null);
				}
			}
		} while (foundLinkees);
		
		// update global minimum
		findNewMin(firstNode);
	}

	/**
	 * Performs the Cascading Cut operation
	 * that is part of a decreaseKey invocation.
	 * 
	 * @param entry The node to be cut from its parent
	 * @return
	 */
	private void nodeCut(FibonacciHNode<T> entry) {
		if (entry != null) {
			// cache the node with the minimum key
			// found during the cascading cat, for the final minimum update
			FibonacciHNode<T> minFound = entry;
			
			double minKey = minFound.getKey();

			// initialize this with the first node to cut
			FibonacciHNode<T> target = entry;
			FibonacciHNode<T> current = null;
			
			do {
				// operate on the last marked node that was found
				// (or the node passed to the function)
				current = target;
	
				// detach node from its parent
				current.parent.degree--;
				
				if (current.parent.degree == 0) {
					current.parent.child = null;
				} else if (current.parent.child == current) {
					current.parent.child = current.leftSibling;
				}
				
				// check if its parent is root
				if (current.parent.parent != null) {
					// parent is not a root, check if it is marked
					if (current.parent.marked) {
						// it is marked, perform another iteration
						target = current.parent;
	
					} else {
						// it is not marked, mark it and enable stop condition
						current.parent.marked = true;
						target = null;
					}
					
				} else {
					// parent is a root, enable stop condition
					target = null;
				}
	
				// update cut node and add it to the root list
				current.parent = null;
				current.marked = false;
				current.unlinkSiblings();
				addRoot(current);
				
				if (current.getKey() < minKey) {
					minFound = current;
					minKey = minFound.getKey();
				}
			} while (target != null);
			
			// update min pointer by comparing it with the top-level node
			// that we cut from its parent
			updateMin(minFound);
		}
	}

	/**
	 * Forcibly empties the Fibonacci Heap.
	 * Used internally to destroy heaps merged into one another.
	 */
	private void empty() {
		minPointer = null;
		heapSize = 0;
	}
	
	/**
	 * Adds a new node to the Fibonacci Heap.
	 * 
	 * @param item The new item
	 * @param key The key associated to the new item
	 * @return The FibonacciHNode instance of the new item
	 */
	public FibonacciHNode<T> insert(T item, double key) {
		FibonacciHNode<T> entry = new FibonacciHNode<T>(item, key);
		
		// add the new item in the root list
		addRoot(entry);
		updateMin(entry);
		
		heapSize++;
		
		return entry;
	}

	/**
	 * Merges this Fibonacci Heap with another one.
	 * IMPORTANT: this operation is destroys the parameter heap,
	 * i.e. after a.meldWith(b), b becomes an empty heap.
	 * 
	 * @param otherHeap The Fibonacci Heap to merge.
	 */
	public void meldWith(FibonacciHeap<T> otherHeap) {
		if (otherHeap != null) {
			// the operation is performed by linking the root lists
			// together; as the root list is referenced by its minimum,
			// retrieve the minimum of the second heap, and add it
			// to this heap together with its siblings
			if (!otherHeap.isEmpty()) {
				FibonacciHNode<T> otherMin = otherHeap.findMin();
				
				addRoot(otherMin);
				updateMin(otherMin);
				
				heapSize += otherHeap.heapSize;
				
				// destroy the old heap because it is dangerous to
				// reference it after this operation.
				otherHeap.empty();
			}

		} else {
			throw new IllegalArgumentException("Null heap.");
		}
	}

	/**
	 * Returns the global minimum of this Fibonacci Heap.
	 * 
	 * @return The global minimum of this Fibonacci Heap.
	 */
	public FibonacciHNode<T> findMin() {
		return minPointer;
	}

	/**
	 * Returns true if this Fibonacci Heap has no nodes,
	 * false otherwise.
	 * 
	 * @return Empty flag of this Fibonacci Heap.
	 */
	public boolean isEmpty() {
		return minPointer == null;
	}

	/**
	 * Finds and delete the node with the minimum key.
	 * 
	 * @return An undetached FibonacciHNode containing the node with the minimum key.
	 */
	public FibonacciHNode<T> deleteMin() {
		FibonacciHNode<T> min = minPointer;
		
		if (min != null) {
			// detach children and siblings from the minimum node,
			// caching them
			FibonacciHNode<T> sibling = min.rightSibling;
			FibonacciHNode<T> oldChild = min.unlinkChildren();
			min.unlinkSiblings();
			
			heapSize--;
	
			if (sibling == min) {
				// the node had no siblings: make a new root list adding its children
				// with addRoot, and rearrange the Fibonacci Heap
				minPointer = null;
				if (oldChild != null) {
					addRoot(oldChild);
					rearrange(oldChild);
				}

			} else {
				// the node had siblings: add its children as siblings so they
				// appear in the root list, and rearrange the Fibonacci Heap
				if (oldChild != null) {
					sibling.linkSibling(oldChild);
				}
				rearrange(sibling);
			}
		}
		return min;
	}

	/**
	 * Decreases key of a node by a given positive difference.
	 * For instance, if a node has key 15.0,
	 * after decreaseKeyBy(node, 2.0) its key will be 15.0-2.0 = 13.0.
	 * 
	 * @param entry The node, the key of which should be decreased.
	 * @param delta The difference.
	 */
	public void decreaseKeyBy(FibonacciHNode<T> entry, double delta) {
		double currentKey = entry.getKey();
		if (delta > 0) {
			double newKey = currentKey - delta;
			doDecreaseKey(entry, newKey);
		} else {
			throw new IllegalArgumentException("Delta must be greater than 0.");
		}
	}
	
	/**
	 * Decreases key of a node to a given new value.
	 * For instance, if a node has key 15.0,
	 * after decreaseKeyTo(node, 2.0) its key will be 2.0.
	 * 
	 * @param entry The node, the key of which should be decreased.
	 * @param newKey The new value for the node key.
	 */
	public void decreaseKeyTo(FibonacciHNode<T> entry, double newKey) {
		double currentKey = entry.getKey();
		if (newKey < currentKey) {
			doDecreaseKey(entry, newKey);
		} else {
			throw new IllegalArgumentException("New key must be lower than old key.");
		}
	}
	
	/**
	 * Internal method to actually decrease key of a node.
	 * 
	 * @param entry The node, the key of which should be decreased.
	 * @param newKey The new value for the node key.
	 */
	private void doDecreaseKey(FibonacciHNode<T> entry, double newKey) {
		entry.key = newKey;
		if (entry.parent == null) {
			updateMin(entry);
		} else if (entry.parent.key > newKey) {
			nodeCut(entry);
		}
	}

	/**
	 * Deletes an arbitrary node from the Fibonacci Heap.
	 * @param entry The node to be deleted
	 * @return
	 */
	public FibonacciHNode<T> delete(FibonacciHNode<T> entry) {
		decreaseKeyTo(entry, Double.NEGATIVE_INFINITY);
		return deleteMin();
	}
	
	public int getHeapSize() {
		return heapSize;
	}
}
